import { DatabaseErrorCode } from '../../constant';
import { getMessage as databaseGetMessage, getMultipleMessage as databaseGetMultipleMessage, getSendingMessageList as databaseGetSendingMessageList, getNormalMsgSeq as databaseGetNormalMsgSeq, updateMessageTimeAndStatus as databaseUpdateMessageTimeAndStatus, updateMessage as databaseUpdateMessage, insertMessage as databaseAddMessage, batchInsertMessageList as databaseBatchInsertMessageList, getMessageList as databaseGetMesageList, getMessageListNoTime as databaseGetMessageListNoTime, searchAllMessageByContentType as databaseSearchAllMessageByContentType, getMsgSeqListByPeerUserID as databaseGetMsgSeqListByPeerUserID, getMsgSeqListBySelfUserID as databaseGetMsgSeqListBySelfUserID, getMsgSeqListByGroupID as databaseGetMsgSeqListByGroupID, updateMessageStatusBySourceID as databaseUpdateMessageStatusBySourceID, } from '../../sqls';
import { converSqlExecResult, convertObjectField, convertToSnakeCaseObject, formatResponse, jsonDecode, } from '../../utils';
import { getInstance } from './instance';
export async function getMessage(messageId) {
    try {
        const db = await getInstance();
        const execResult = databaseGetMessage(db, messageId);
        if (execResult.length === 0) {
            return formatResponse('', DatabaseErrorCode.ErrorNoRecord, `no message with id ${messageId}`);
        }
        return formatResponse(converSqlExecResult(execResult[0], 'CamelCase', [
            'isRead',
            'isReact',
            'isExternalExtensions',
        ])[0]);
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function getMultipleMessage(messageIdsStr) {
    try {
        const db = await getInstance();
        const execResult = databaseGetMultipleMessage(db, jsonDecode(messageIdsStr, []));
        return formatResponse(converSqlExecResult(execResult[0], 'CamelCase', [
            'isRead',
            'isReact',
            'isExternalExtensions',
        ]));
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function getSendingMessageList() {
    try {
        const db = await getInstance();
        const execResult = databaseGetSendingMessageList(db);
        return formatResponse(converSqlExecResult(execResult[0], 'CamelCase', [
            'isRead',
            'isReact',
            'isExternalExtensions',
        ]));
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function getNormalMsgSeq() {
    try {
        const db = await getInstance();
        const execResult = databaseGetNormalMsgSeq(db);
        return formatResponse(execResult[0]?.values[0]?.[0]);
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function updateMessageTimeAndStatus(clientMsgID, serverMsgID, sendTime, status) {
    try {
        const db = await getInstance();
        const execResult = databaseUpdateMessageTimeAndStatus(db, clientMsgID, serverMsgID, sendTime, status);
        return formatResponse(execResult);
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function updateMessage(clientMsgId, messageStr) {
    try {
        const db = await getInstance();
        const message = convertToSnakeCaseObject(convertObjectField(JSON.parse(messageStr), { groupName: 'name' }));
        const execResult = databaseUpdateMessage(db, clientMsgId, message);
        const modifed = db.getRowsModified();
        if (modifed === 0) {
            throw 'updateMessage no record updated';
        }
        return formatResponse(execResult);
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function updateColumnsMessage(clientMsgId, messageStr) {
    try {
        const db = await getInstance();
        const execResult = databaseUpdateMessage(db, clientMsgId, JSON.parse(messageStr));
        const modifed = db.getRowsModified();
        if (modifed === 0) {
            throw 'updateMessage no record updated';
        }
        return formatResponse(execResult);
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function insertMessage(messageStr) {
    try {
        const db = await getInstance();
        const message = convertToSnakeCaseObject(JSON.parse(messageStr));
        const execResult = databaseAddMessage(db, message);
        return formatResponse(execResult);
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function batchInsertMessageList(messageListStr) {
    try {
        const db = await getInstance();
        const messageList = JSON.parse(messageListStr).map((v) => convertToSnakeCaseObject(v));
        const execResult = databaseBatchInsertMessageList(db, messageList);
        return formatResponse(execResult);
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function getMessageListNoTime(sourceID, sessionType, count, isReverse = false, loginUserID = '') {
    try {
        const db = await getInstance();
        const execResult = databaseGetMessageListNoTime(db, sourceID, sessionType, count, isReverse, loginUserID);
        return formatResponse(converSqlExecResult(execResult[0], 'CamelCase', [
            'isRead',
            'isReact',
            'isExternalExtensions',
        ]));
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function getMessageList(sourceID, sessionType, count, startTime, isReverse = false, loginUserID) {
    try {
        const db = await getInstance();
        const execResult = databaseGetMesageList(db, sourceID, sessionType, count, startTime, isReverse, loginUserID);
        return formatResponse(converSqlExecResult(execResult[0], 'CamelCase', [
            'isRead',
            'isReact',
            'isExternalExtensions',
        ]));
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function searchAllMessageByContentType(contentType) {
    try {
        const db = await getInstance();
        const execResult = databaseSearchAllMessageByContentType(db, contentType);
        return formatResponse(converSqlExecResult(execResult[0], 'CamelCase', [
            'isRead',
            'isReact',
            'isExternalExtensions',
        ]));
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function getMsgSeqListByPeerUserID(userID) {
    try {
        const db = await getInstance();
        const execResult = databaseGetMsgSeqListByPeerUserID(db, userID);
        const converedResult = converSqlExecResult(execResult[0], 'CamelCase');
        return formatResponse(converedResult.map(item => {
            return item.seq;
        }));
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function getMsgSeqListBySelfUserID(userID) {
    try {
        const db = await getInstance();
        const execResult = databaseGetMsgSeqListBySelfUserID(db, userID);
        const converedResult = converSqlExecResult(execResult[0], 'CamelCase');
        return formatResponse(converedResult.map(item => {
            return item.seq;
        }));
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function getMsgSeqListByGroupID(groupID) {
    try {
        const db = await getInstance();
        const execResult = databaseGetMsgSeqListByGroupID(db, groupID);
        const converedResult = converSqlExecResult(execResult[0], 'CamelCase');
        return formatResponse(converedResult.map(item => {
            return item.seq;
        }));
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
export async function updateMessageStatusBySourceID(sourceID, status, sessionType, loginUserID) {
    try {
        const db = await getInstance();
        databaseUpdateMessageStatusBySourceID(db, sourceID, status, sessionType, loginUserID);
        const modifed = db.getRowsModified();
        if (modifed === 0) {
            // throw 'updateMessageStatusBySourceID no record updated';
        }
        return formatResponse(modifed);
    }
    catch (e) {
        console.error(e);
        return formatResponse(undefined, DatabaseErrorCode.ErrorInit, JSON.stringify(e));
    }
}
